@@ -45,6 +45,7 @@ namespace PoDoFo {
 
 class PdfDictionary;
 class PdfFont;
+class PdfObject;
 
 /** 
  * A PdfEncoding is in PdfFont to transform a text string
@@ -62,10 +63,10 @@ class PODOFO_API PdfEncoding {
      *  \param nFirstChar the first supported character code 
      *                    (either a byte value in the current encoding or a unicode value)
      *  \param nLastChar the last supported character code, must be larger than nFirstChar 
--     *                    (either a byte value in the current encoding or a unicode value)
+     *                    (either a byte value in the current encoding or a unicode value)
      *
      */
-    PdfEncoding( int nFirstChar, int nLastChar );
+    PdfEncoding( int nFirstChar, int nLastChar, PdfObject* = NULL );
 
     /** Get a unique ID for this encoding
      *  which can used for comparisons!
@@ -165,7 +166,7 @@ class PODOFO_API PdfEncoding {
      *
      *  \returns an unicode PdfString.
      */
-    virtual PdfString ConvertToUnicode( const PdfString & rEncodedString, const PdfFont* pFont ) const = 0;
+    virtual PdfString ConvertToUnicode( const PdfString & rEncodedString, const PdfFont* pFont ) const;
 
     /** Convert a unicode PdfString to a string encoded with this encoding.
      *
@@ -175,7 +176,7 @@ class PODOFO_API PdfEncoding {
      *  \returns an encoded PdfRefCountedBuffer. The PdfRefCountedBuffer is treated as a series of bytes
      *           and is allowed to have 0 bytes. The returned buffer must not be a unicode string.
      */
-    virtual PdfRefCountedBuffer ConvertToEncoding( const PdfString & rString, const PdfFont* pFont ) const = 0;
+    virtual PdfRefCountedBuffer ConvertToEncoding( const PdfString & rString, const PdfFont* pFont ) const;
 
     virtual bool IsAutoDelete() const = 0;
 
@@ -219,9 +220,22 @@ class PODOFO_API PdfEncoding {
      */
     virtual pdf_utf16be GetCharCode( int nIndex ) const = 0;
 
+ protected:
+    bool m_bToUnicodeIsLoaded;  ///< If true, ToUnicode has been parse
+                             
  private:
     int     m_nFirstChar;   ///< The first defined character code
     int     m_nLastChar;    ///< The last defined character code
+    PdfObject* m_pToUnicode;    ///< Pointer to /ToUnicode object, if any
+ protected:
+    std::map<pdf_utf16be, pdf_utf16be> m_toUnicode;
+ private:
+    
+    /** Parse the /ToUnicode object
+    */
+    void ParseToUnicode();
+    pdf_utf16be GetUnicodeValue( pdf_utf16be ) const;
+    pdf_utf16be GetCIDValue( pdf_utf16be ) const;
 };
 
 // -----------------------------------------------------
@@ -503,6 +517,9 @@ class PODOFO_API PdfDocEncoding : public PdfSimpleEncoding {
  * contents on PDF pages.
  *
  * It is also called CP-1252 encoding.
+ * This class may be used as base for derived encodings.
+ *
+ * \see PdfWin1250Encoding
  *
  * Do not allocate this class yourself, as allocations
  * might be expensive. Try using PdfFont::WinAnsiEncoding.
@@ -533,6 +550,17 @@ class PODOFO_API PdfWinAnsiEncoding : public PdfSimpleEncoding {
      */
     virtual const pdf_utf16be* GetToUnicodeTable() const;
 
+    /** Add this encoding object to a dictionary
+     *  usually be adding an /Encoding key in font dictionaries.
+     *  
+     *  This method generates array of differences into /Encoding
+     *  dictionary if called from derived class with
+     *  different unicode table.
+     *
+     *  \param rDictionary add the encoding to this dictionary
+     */
+    virtual void AddToDictionary( PdfDictionary & rDictionary ) const;
+
  private:
     static const pdf_utf16be s_cEncoding[256]; ///< conversion table from WinAnsiEncoding to UTF16
 
@@ -712,6 +740,66 @@ class PODOFO_API PdfZapfDingbatsEncoding : public PdfSimpleEncoding {
 
 };
 
+/**
+* WINDOWS-1250 encoding
+*/
+class PODOFO_API PdfWin1250Encoding : public PdfWinAnsiEncoding
+{
+ public:
+   
+    /** Create a new PdfWin1250Encoding
+     */
+    PdfWin1250Encoding()
+    {
+    }
+
+ protected:
+
+    /** Gets a table of 256 short values which are the 
+     *  big endian unicode code points that are assigned
+     *  to the 256 values of this encoding.
+     *
+     *  This table is used internally to convert an encoded
+     *  string of this encoding to and from unicode.
+     *
+     *  \returns an array of 256 big endian unicode code points
+     */
+    virtual const pdf_utf16be* GetToUnicodeTable() const;
+
+ private:
+    static const pdf_utf16be s_cEncoding[256]; ///< conversion table from Win1250Encoding to UTF16
+};
+
+/**
+* ISO-8859-2 encoding
+*/
+class PODOFO_API PdfIso88592Encoding : public PdfWinAnsiEncoding
+{
+ public:
+   
+    /** Create a new PdfIso88592Encoding
+     */
+    PdfIso88592Encoding()
+    {
+    }
+
+ protected:
+
+    /** Gets a table of 256 short values which are the 
+     *  big endian unicode code points that are assigned
+     *  to the 256 values of this encoding.
+     *
+     *  This table is used internally to convert an encoded
+     *  string of this encoding to and from unicode.
+     *
+     *  \returns an array of 256 big endian unicode code points
+     */
+    virtual const pdf_utf16be* GetToUnicodeTable() const;
+
+ private:
+    static const pdf_utf16be s_cEncoding[256]; ///< conversion table from Iso88592Encoding to UTF16
+};
+
 }; /* namespace PoDoFo */
 
 
